@@ -52,7 +52,7 @@ def _jump_iterator(_graph, _kind, _entity_id_iterator, _traversal, _neighbors_ca
 
 def check_base_node(entity):
     if not(isinstance(entity, Node)):
-        raise GrapheekInvalidDataTypeException('%s is not a node')
+        raise GrapheekInvalidDataTypeException('%s is not a node' % (entity,))
 
 
 def check_and_get_count(args):
@@ -484,14 +484,18 @@ class EntityIterator(object):
         return EntityAggregate(self._graph, self._src_kind, self, pair_iterator=None, agg=agg, asc=asc)
 
     def call(self, _fname, *aliases, **params):
+        contexts = []
         for entity_id in self._iterate():  # Iterating updates the context ...
-            self._graph._call_server_script(self._context, _fname, *aliases, **params)
+            contexts.append(self._context)
+        self._graph._call_server_script(contexts, _fname, *aliases, **params)
 
     def request(self, _fname, *aliases, **params):
+        contexts = []
         # Same as call but returns results of each call
         results = []
         for entity_id in self._iterate():  # Iterating updates the context ...
-            results.append(self._graph._call_server_script(self._context, _fname, *aliases, **params))
+            contexts.append(self._context)
+        results.append(self._graph._call_server_script(contexts, _fname, *aliases, **params))
         return results
 
     def dot(self, node_label=None, edge_label=None, limit=100):
@@ -801,13 +805,18 @@ class Edge(Entity):
 
 class BaseGraph(GraphReadWrite):
 
-    def __del__(self):  # pragma : no cover
-        # ensure connection is closed
+    def close(self):
+        if not self._closed:
+            self._db_close()
+            self._closed = True
+
+    def __getattribute__(self, attr):
         try:
-            if not self._closed:
-                self._db_close()
+            if object.__getattribute__(self, '_closed'):
+                raise GrapheekDataException('Graph is closed')
         except AttributeError:
-            pass
+            pass  # no _closed attribute -> no definitely not closed (not even open, in fact)
+        return object.__getattribute__(self, attr)
 
     def __initialize__(self):
         # "Rebuilding" indexes :
@@ -825,16 +834,16 @@ class BaseGraph(GraphReadWrite):
                 fields, filter_tuples = self._get(txn, build_key(METADATA_VERTEX_INDEX_FIELDS_PREFIX, index_id))
                 filters = dict(filter_tuples)
                 if fields != UNDEFINED:
-                    prefix = build_key(METADATA_VERTEX_INDEX_PREFIX, index_id)
-                    self._node_indexes.append(ExactIndex(KIND_VERTEX, prefix, *fields, **filters))
+                    prefix = build_key(KIND_INDEX, KIND_VERTEX, index_id)
+                    self._node_indexes.append(ExactIndex(self, KIND_VERTEX, prefix, *fields, **filters))
             #   2nd edge indexes (~ same code, but I'm not sure it makes sense to factor...)
             index_count = self._get(txn, METADATA_EDGE_INDEX_COUNTER)
             for index_id in range(0, index_count):
-                fields, filter_tuples = self._get(txn, build_key(METADATA_VERTEX_INDEX_FIELDS_PREFIX, index_id))
+                fields, filter_tuples = self._get(txn, build_key(METADATA_EDGE_INDEX_FIELDS_PREFIX, index_id))
                 filters = dict(filter_tuples)
                 if fields != UNDEFINED:
-                    prefix = build_key(METADATA_VERTEX_INDEX_PREFIX, index_id)
-                    self._edge_indexes.append(ExactIndex(KIND_VERTEX, prefix, *fields, **filters))
+                    prefix = build_key(KIND_INDEX, KIND_EDGE, index_id)
+                    self._edge_indexes.append(ExactIndex(self, KIND_EDGE, prefix, *fields, **filters))
             self._transaction_commit(txn)
         except Exception, e:  # pragma : no cover
             self._transaction_rollback(txn)
@@ -1056,13 +1065,18 @@ class BaseGraph(GraphReadWrite):
             node_count = len(node_defns)
             last_node_id = self._update_inc(txn, METADATA_VERTEX_COUNTER, node_count) - 1
             first_node_id = last_node_id - node_count + 1
-            for node_id, data in zip(range(first_node_id, last_node_id + 1), node_defns):
+            range_ids = range(first_node_id, last_node_id + 1)
+            for node_id, data in zip(range_ids, node_defns):
                 operation.merge(self._add_node(txn, data, node_id))
             operation.apply(txn, self)
+            # now updating indexes :
+            self._bulk_add_to_all_entity_indexes(txn, KIND_VERTEX, range_ids)
+            # update denormalized node count
             self._node_count += node_count
+            # all is ok, commit :)
             self._transaction_commit(txn)
             if not(silent):
-                res = [Node(node_id, self) for node_id in range(first_node_id, last_node_id + 1)]
+                res = [Node(node_id, self) for node_id in range_ids]
                 return res
         except Exception, e:
             self._transaction_rollback(txn)
@@ -1150,14 +1164,19 @@ class BaseGraph(GraphReadWrite):
             edge_count = len(edge_defns)
             last_edge_id = self._update_inc(txn, METADATA_EDGE_COUNTER, edge_count) - 1
             first_edge_id = last_edge_id - edge_count + 1
-            for edge_id, edge_defn in zip(range(first_edge_id, last_edge_id + 1), edge_defns):
+            range_ids = range(first_edge_id, last_edge_id + 1)
+            for edge_id, edge_defn in zip(range_ids, edge_defns):
                 source, target, data = edge_defn
                 operation.merge(self._add_edge(txn, source, target, data, edge_id=edge_id))
             operation.apply(txn, self)
+            # now updating indexes :
+            self._bulk_add_to_all_entity_indexes(txn, KIND_EDGE, range_ids)
+            # update denormalized node count
             self._edge_count += edge_count
+            # all is ok, commit :)
             self._transaction_commit(txn)
             if not(silent):
-                return [Edge(edge_id, self) for edge_id in range(first_edge_id, last_edge_id + 1)]
+                return [Edge(edge_id, self) for edge_id in range_ids]
         except Exception, e:
             self._transaction_rollback(txn)
             raise GrapheekDataException(repr(e))
@@ -1347,6 +1366,11 @@ class BaseGraph(GraphReadWrite):
         for index in indexes:
             index.add(txn, entity_id, data)
 
+    def _bulk_add_to_all_entity_indexes(self, txn, kind, entity_ids):
+        indexes = self._node_indexes if kind == KIND_VERTEX else self._edge_indexes
+        for index in indexes:
+            index.bulk_add(txn, iter(entity_ids))
+
     def _remove_from_all_entity_indexes(self, txn, kind, entity_id):
         indexes = self._node_indexes if kind == KIND_VERTEX else self._edge_indexes
         for index in indexes:
@@ -1396,7 +1420,7 @@ class BaseGraph(GraphReadWrite):
             # that will (should ;)) speedup query
             id_iterator = self._optimizer.get_kind_ids(txn, _kind)
             # Building index :
-            index.build(txn, id_iterator)
+            index.bulk_add(txn, id_iterator)
             # Everything seems to be ok, adding index to proper list
             entity_indexes.append(index)
             self._transaction_commit(txn)
