@@ -1,9 +1,6 @@
 # -*- coding:utf-8 -*-
 
-try:
-    import ujson as json
-except ImportError:  # pragma : no cover
-    import json
+import msgpack
 
 from kyotocabinet import DB
 
@@ -57,17 +54,17 @@ class KyotoCabinetGraph(BaseGraph):
         raw_data = self._db.get(key)
         if raw_data is None:
             return UNDEFINED  # Not returning None, as None is a valid value
-        return json.loads(raw_data)
+        return msgpack.loads(raw_data)
 
     def _bulk_get(self, txn, keys):
         result = {}
         key_raw_datas = self._db.get_bulk(keys)
         for key, raw_data in key_raw_datas.items():
-            result[key] = json.loads(raw_data)
+            result[key] = msgpack.loads(raw_data)
         return result
 
     def _set(self, txn, key, value):
-        res = self._db.set(key, json.dumps(value))
+        res = self._db.set(key, msgpack.dumps(value))
         if not(res):  # pragma : no cover
             raise GrapheekDataKyotoCabinetException('KyotoCabinet : error while saving')
         return res
@@ -75,7 +72,7 @@ class KyotoCabinetGraph(BaseGraph):
     def _bulk_set(self, txn, updates):
         dic = {}
         for key, value in updates.items():
-            dic[key] = json.dumps(value)
+            dic[key] = msgpack.dumps(value)
         res = self._db.set_bulk(dic)
         if res == -1:  # pragma : no cover
             raise GrapheekDataKyotoCabinetException('KyotoCabinet : error while saving')
@@ -101,7 +98,7 @@ class KyotoCabinetGraph(BaseGraph):
     # overriding list methods
     # looks like a bucket of hacks, and yes indeed it is :)
     # btw, it REALLY improves performance if we compare to default implementation which,
-    # in the case of KyotoCabinet would involve json deserialization followed by a serialization
+    # in the case of KyotoCabinet would involve msgpack deserialization followed by a serialization
 
     def _init_lst(self, txn, key):
         res = self._db.set(key, '')
