@@ -62,47 +62,50 @@ def runserver(_address, _backend, _scripts, _context, **params):
     # -- END
     GraphClass = getattr(class_module, class_name)
     g = GraphClass(**params)
-    # Setting script module paths :
-    g.setup_server_scripts(*_scripts.split(':'))
-    # --
-    if _context is None:  # pragma : no cover
-        _context = zmq.Context()
-    socket = _context.socket(zmq.REP)
-    socket.bind(_address)
-    # -----------------------------------------
-    stop = False
-    inproc = _address.startswith('inproc://')
-    while not(stop):  # pragma : no cover
-        try:
-            # Wait for next request from client
-            raw = socket.recv()
-            # if address is inproc, it means that the "server" greenlet had been start from another greenlet
-            # -> the parent needs a way to stop imperatively the "server" greenlet, thus the following line :
-            if inproc:
-                if raw == 'stop':
-                    socket.send('ok')
-                    break
-            # Handling request :
-            data = msgpack.loads(raw)
-            obj = g
-            for item in data:
-                method_name, args, kwargs = item
-                method = getattr(obj, method_name)
-                obj = method(*args, **kwargs)
-            result = marshall(obj)
-            socket.send(msgpack.dumps(result))
-        except KeyboardInterrupt:
-            stop = True
-        except:
-            result = {
-                '__': 'x',
-                'd': traceback.format_exc()
-            }
+    try:
+        # Setting script module paths :
+        g.setup_server_scripts(*_scripts.split(':'))
+        # --
+        if _context is None:  # pragma : no cover
+            _context = zmq.Context()
+        socket = _context.socket(zmq.REP)
+        socket.bind(_address)
+        # -----------------------------------------
+        stop = False
+        inproc = _address.startswith('inproc://')
+        while not(stop):  # pragma : no cover
             try:
+                # Wait for next request from client
+                raw = socket.recv()
+                # if address is inproc, it means that the "server" greenlet had been start from another greenlet
+                # -> the parent needs a way to stop imperatively the "server" greenlet, thus the following line :
+                if inproc:
+                    if raw == 'stop':
+                        socket.send('ok')
+                        break
+                # Handling request :
+                data = msgpack.loads(raw)
+                obj = g
+                for item in data:
+                    method_name, args, kwargs = item
+                    method = getattr(obj, method_name)
+                    obj = method(*args, **kwargs)
+                result = marshall(obj)
                 socket.send(msgpack.dumps(result))
+            except KeyboardInterrupt:
+                stop = True
             except:
-                pass  # The server must not fail
-                # TODO : At least log an info
+                result = {
+                    '__': 'x',
+                    'd': traceback.format_exc()
+                }
+                try:
+                    socket.send(msgpack.dumps(result))
+                except:
+                    pass  # The server must not fail
+                    # TODO : At least log an info
+    finally:
+        g.close()
 
 
 def read_config_file(path):
